import os
import sys
import textwrap
from dotenv import load_dotenv


class Configurator:
    def __init__(self):
        self.env_exists = load_dotenv()
        self.geocode_auth = os.getenv('GEOCODE_AUTH') if self.env_exists else ''
        self.datahub_api_key = os.getenv('DATAHUB_API_KEY') if self.env_exists else ''
        self.datahub_secret = os.getenv('DATAHUB_SECRET') if self.env_exists else ''

    def initial_setup(self):
        """
        Takes 3 inputs from stdin for the geocode.xyz auth token, Met Office Weather DataHub API key and secret,
        which is then written out to a new .env file to use with the program.
        """
        confirmation = input(textwrap.dedent(f"""\
        WARNING: this will truncate the file at {os.path.realpath('./.env')} if it exists.
        Do you wish to continue? (Y/n)
        """))

        if confirmation.lower() != "y":
            print("Exiting without saving...")
            sys.exit(0)

        geocode_auth = input("Please enter your geocode.xyz auth token (or press enter to skip): ")
        if geocode_auth != '':
            self.geocode_auth = geocode_auth

        datahub_api_key = input("Please enter your Met Office Weather DataHub API key (or press enter to skip): ")
        if datahub_api_key != '':
            self.datahub_api_key = datahub_api_key

        datahub_secret = input("Please enter your Met Office Weather DataHub API secret (or press enter to skip): ")
        if datahub_secret != '':
            self.datahub_secret = datahub_secret

        self.generate_dotenv()

    def generate_dotenv(self):
        mode = 'w'
        if not self.env_exists:
            mode = 'x'
        with open('./.env', mode) as file:
            file.write(textwrap.dedent(f"""\
            # Environment variables for metoffice-weather-cli
            GEOCODE_AUTH={self.geocode_auth}
            DATAHUB_API_KEY={self.datahub_api_key}
            DATAHUB_SECRET={self.datahub_secret}
            """))
            print(f"Saved dotenv file at {os.path.realpath('./.env')}")


if __name__ == '__main__':
    configurator = Configurator()
    configurator.initial_setup()



import pytest
from metoffice_weather_cli.weathercodes import decode_weather_type, decode_uv_index


def test_decode_uv_index():
    assert 'Low exposure' == decode_uv_index(1)
    assert 'Medium exposure' == decode_uv_index(5)
    assert 'High exposure' == decode_uv_index(6)
    assert 'Extremely high exposure' == decode_uv_index(13)



import os
from dotenv import load_dotenv, find_dotenv
import http.client
import urllib.parse
import json
from .weathercodes import decode_weather_type, decode_uv_index


def get_geolocation(location: str):
    load_dotenv()
    # connect to geocoding API
    _check_env_variable_exists('GEOCODE_AUTH')

    connection = http.client.HTTPSConnection("geocode.xyz")
    params = urllib.parse.urlencode({
        'auth': os.getenv('GEOCODE_AUTH'),
        'locate': location,
        'json': 1
    })

    connection.request("GET", "/?{}".format(params))

    geocode_res = connection.getresponse()
    geocode_json = json.loads(geocode_res.read())  # reads JSON file from response

    if 'error' in geocode_json:
        raise KeyError("API error - your request produced no suggestions.")
    return geocode_json


def get_weather_info(city: str, country: str, latt: float, longt: float):
    load_dotenv()
    _check_env_variable_exists('DATAHUB_API_KEY')
    _check_env_variable_exists('DATAHUB_SECRET')

    # connect to Weather DataHub
    datahub_conn = http.client.HTTPSConnection("rgw.5878-e94b1c46.eu-gb.apiconnect.appdomain.cloud")

    datahub_headers = {
        'X-IBM-Client-Id': os.getenv('DATAHUB_API_KEY'),
        'X-IBM-Client-Secret': os.getenv('DATAHUB_SECRET'),
        'accept': "application/json"
    }

    datahub_params = urllib.parse.urlencode({
        'excludeParameterMetadata': 'true',
        'includeLocationName': 'true',
        'latitude': str(latt),
        'longitude': str(longt)
    })

    datahub_conn.request('GET',
                         '/metoffice/production/v0/forecasts/point/daily?{}'.format(datahub_params),
                         headers=datahub_headers
                         )

    datahub_res = datahub_conn.getresponse()
    datahub_json = json.loads(datahub_res.read())

    if 'features' not in datahub_json:
        raise KeyError("Met Office API error - check that your API key and secret are correct.")
    time_series = datahub_json['features'][0]['properties']['timeSeries'][1]

    weather_data = {
        "City": city,
        "Country": country,
        "TimeOfModel": time_series['time'],
        "WeatherType": decode_weather_type(time_series['daySignificantWeatherCode']),
        "MaxTemperature": time_series['dayUpperBoundMaxTemp'],  # degrees Celsius
        "MinTemperature": time_series['dayLowerBoundMaxTemp'],
        "ChanceOfPrecipitation": time_series['dayProbabilityOfPrecipitation'],  # %
        "WindSpeed": time_series['midday10MWindSpeed'],  # m/s
        "MaxUvIndex": time_series['maxUvIndex']
    }
    return weather_data


def _check_env_variable_exists(env_variable):
    if os.getenv(env_variable) is None or os.getenv(env_variable) == '':
        raise ValueError(f"'{env_variable}' environment variable not set, check your .env file")


def print_results(data):
    # todo: control which options are printed as determined by args
    print(f"Weather for {data['City']}, {data['Country']}")
    print(f"Weather: {data['WeatherType']}")
    print(f"Max temp: {round(data['MaxTemperature'])} degrees Celsius")
    print(f"Min temp: {round(data['MinTemperature'])} degrees Celsius")
    print(f"Chance of precipitation: {data['ChanceOfPrecipitation']}%")
    print(f"Wind speed: {round(float(data['WindSpeed'] / 0.44704), 1)} mph")
    print(f"Peak UV Index: {data['MaxUvIndex']} ({decode_uv_index(data['MaxUvIndex'])})")


if __name__ == '__main__':
    load_dotenv(find_dotenv())
    print("Please enter desired city:")
    stdin = input("> ")
    geolocation = get_geolocation(stdin)
    result = get_weather_info(
        geolocation['standard']['city'],
        geolocation['standard']['countryname'],
        geolocation['latt'],
        geolocation['longt']
    )
    print_results(result)



weathercodes = {
    'NA': 'Not available',
    '0': 'Clear night',
    '1': 'Sunny day',
    '2': 'Partly cloudy (night)',
    '3': 'Partly cloudy (day)',
    '4': 'Not used',
    '5': 'Mist',
    '6': 'Fog',
    '7': 'Cloudy',
    '8': 'Overcast',
    '9': 'Light rain shower (night)',
    '10': 'Light rain shower (day)',
    '11': 'Drizzle',
    '12': 'Light rain',
    '13': 'Heavy rain shower (night)',
    '14': 'Heavy rain shower (day)',
    '15': 'Heavy rain',
    '16': 'Sleet shower (night)',
    '17': 'Sleet shower (day)',
    '18': 'Sleet',
    '19': 'Hail shower (night)',
    '20': 'Hail shower (day)',
    '21': 'Hail',
    '22': 'Light snow shower (night)',
    '23': 'Light snow shower (day)',
    '24': 'Light snow',
    '25': 'Heavy snow shower (night)',
    '26': 'Heavy snow shower (day)',
    '27': 'Heavy snow',
    '28': 'Thunder shower (night)',
    '29': 'Thunder shower (day)',
    '30': 'Thunder'
}


def decode_weather_type(code):
    try:
        return weathercodes[str(code)]
    except KeyError:
        return 'NA'


def decode_uv_index(uv_index):
    if uv_index in range(3):
        return 'Low exposure'
    elif uv_index in range(3, 6):
        return 'Medium exposure'
    elif uv_index in range(6, 8):
        return 'High exposure'
    elif uv_index in range(8, 11):
        return 'Very high exposure'
    else:
        return 'Extremely high exposure'



import argparse
import sys
from .configurator import Configurator
from . import weather
from dotenv import load_dotenv


def run():
    parser = argparse.ArgumentParser(
        prog="weather-cli",
        description="A CLI tool for fetching and printing weather data"
    )
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-c', '--config',
                       action='store_true',
                       help="""Configures the .env file with your geocode.xyz auth token,
                       Met Office Weather DataHub API key and secret"""
                       )
    group.add_argument('-s', '--search',
                       action='store',
                       nargs=1,
                       type=str,
                       metavar='<location>',
                       help="Fetch and print the weather given the name of a city or town"
                       )
    group.add_argument('-g', '--geocode',
                       action='store',
                       nargs=1,
                       type=str,
                       metavar='<location>',
                       help="Fetch and print the latitude and longitude coordinates of a city or town"
                       )
    group.add_argument('-w', '--weather',
                       action='store',
                       nargs=2,
                       type=float,
                       metavar=('<latitude>', '<longitude>'),
                       help="Fetch and print the weather at a set of latitude and longitude coordinates.",
                       )

    args = parser.parse_args()
    if args.config and len(sys.argv) > 1:
        cfg = Configurator()
        print("Running initial setup...")
        cfg.initial_setup()
        sys.exit(0)
    else:
        load_dotenv()

    if args.search is not None:
        geolocation = weather.get_geolocation(args.search)
        data = weather.get_weather_info(
            geolocation['standard']['city'],
            geolocation['standard']['countryname'],
            geolocation['latt'],
            geolocation['longt']
        )
        weather.print_results(data)

    elif args.geocode is not None:
        geolocation = weather.get_geolocation(args.search)
        print(f"{args.geocode[0]} is located at {geolocation['latt']}, {geolocation['longt']}")

    elif args.weather is not None:
        data = weather.get_weather_info(
            str(args.weather[0]),
            str(args.weather[1]),
            args.weather[0],
            args.weather[1])
        weather.print_results(data)

    else:
        parser.print_help()
